<?php
/**
 * @package FlashchatPlugin
 */
/*
Plugin Name: Flashchat App
Description: Flashchat App Plugin integrates your WooCommerce e-shop with Flashchat, automatically providing you with an all-in-one Messenger Commerce Platform.
Version: 1.2.0
Author: Flashchat
Author URI: https://flashchat.ai/
License: GPLv2 or later
*/

/*
Flashchat App Plugin is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.
 
Flashchat App Plugin is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with Flashchat App Plugin. If not, see https://www.gnu.org/licenses/gpl-2.0.html.
*/

if (! defined( 'ABSPATH' ) ) {
    die;
}

/**
 * Check if WooCommerce is active
 **/
if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
    
    // Woocommerce version > 3.0.0
    if ( version_compare( get_option( 'woocommerce_version' ), '3.0.0', '<' ) ) {
        die;
    }
    
    // include and instantiate Flashchat plugin class
    include ( plugin_dir_path( __FILE__ ) . 'includes/class-flashchat-plugin.php');
    if (class_exists( 'FlashchatPlugin' ) ) {
        $flashchatPlugin = new FlashchatPlugin();
    }

    // activation, deactivation, uninstall 
    register_activation_hook( __FILE__, array ( $flashchatPlugin, 'activate' ) );
    register_deactivation_hook( __FILE__, array ( $flashchatPlugin, 'deactivate' ) );
    register_uninstall_hook( __FILE__, 'FlashchatPlugin::uninstall' );

    // Add admin notice right after activation
    add_action( 'admin_notices', array ( $flashchatPlugin, 'admin_notice_after_activation' ) );

    // enqueue flashchat scripts for the provide features
    add_action('wp_enqueue_scripts', array ( $flashchatPlugin, 'enqueue_flashchat' ));
    
    // mount custom endpoints for cart creation and cart retrieval
    add_action('rest_api_init', array ( $flashchatPlugin, 'registerFlashchatCartEndpoints' ));

    // mount custom Woocommerce endpoints from WC_API_Flashchat class for checkingorder placement and posting account id
    add_action( 'woocommerce_api_loaded', function() {
        include_once( 'includes/class-wc-api-flashchat.php' );
    });
    add_filter( 'woocommerce_api_classes', function( $classes ) {
        $classes[] = 'WC_API_Flashchat';
        return $classes;
    });

    // register Flashchat Plugin Settings
    add_action( 'admin_init', function() {
        add_option('account_id', null);
        register_setting( 'myplugin_options_group', 'myplugin_option_name', 'myplugin_callback' );
    });
    add_action('admin_menu', array ( $flashchatPlugin, 'flashchat_register_settings_page' ));

    $plugin = plugin_basename( __FILE__ );
    add_filter( "plugin_action_links_$plugin", function($links) {
        $settings_link = '<a href="options-general.php?page=flashchat_app">' . __( 'Settings' ) . '</a>';
        array_push( $links, $settings_link );
        return $links;
    });
    
    // add customerId as meta data to check abandoned cart
    add_action('woocommerce_checkout_create_order', array ( $flashchatPlugin, 'before_checkout_create_order_add_customer_id' ), 20, 2);

    // handle _customerId as custom query var
    add_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array ( $flashchatPlugin, 'handle_custom_query_var_customer_id' ), 10, 2 );
}
