<?php
/**
 * Class that contains main plugin functionality
 **/
class FlashchatPlugin {
    function __construct() {}

    // invoked from activation hook
    function activate() {
        /* Create transient data for displaying admin notice message*/
        set_transient( 'flashchat-admin-notice-after-activation', true, 5 );
    }

    // invoked from deactivation hook
    function deactivate() {
        // dequeue flashchatInjector on deactivation
        wp_dequeue_script( 'flashchat-script', plugins_url( '/js/flashchatInjector.js', __FILE__ ));
    }

    public static function uninstall() {
        // delete option used to store Flashchat account id
        delete_option( 'account_id');
    }

    // enqueues flashchatInjector.js which will deliver script implementing the features in front end
    function enqueue_flashchat() {
        // load script from file only if option account_id exists
        $accountId = get_option('account_id'); 
        if (!empty($accountId)) {
            $params = array(
                'accountId' => get_option( 'account_id'),
                'storeUrl' => home_url(),
            );
            wp_enqueue_script( 'flashchat-script', plugins_url( '/js/flashchatInjector.js', __FILE__ ));
            wp_localize_script( 'flashchat-script', 'ScriptParams', $params );
        }
    }

    // admin notice right after activation 
    function admin_notice_after_activation() {
        /* Check transient, if available display notice */
        if( get_transient( 'flashchat-admin-notice-after-activation' ) ){
            $imgUrl = plugins_url( 'assets/rocket.png', __FILE__ );
            ?>
                <div class="updated notice is-dismissible" style="min-height: 80px; ">
                    <div style="float: left; margin-top: 6.5px; margin-left: 6.5;">
                        <img src="<?php echo $imgUrl; ?>" width="60" height="60"/>
                    </div>
                    <div>
                    <p><strong>Welcome to Flashchat!</strong> - Click Settings to setup your plugin:</p> 
                    <p><a href="options-general.php?page=flashchat_app"
                        style ="background-color: #007aff; color: white; border: 1px solid #0087be; height: auto; padding: .8em 1.8em; text-decoration: none;" type="submit">
                        Settings
                        </a>
                        </p>
                    </div>
                </div>
            <?php
            /* Delete transient, display this notice until integration is completed. */
            delete_transient( 'flashchat-admin-notice-after-activation' );
        }
    }

    // renders Flashchat Plugin Settings page
    static function flashchat_settings_page() {
        $imgUrl = plugins_url( 'assets/rocket.png', __FILE__ );
        $flashchatCRM = 'https://crm.flashchat.ai/api'; 
        $request_url = $flashchatCRM . '/woocommerce?shop=' . home_url();
    ?>
        <div style="height: 400px; margin-top: 30px;">
            <div style="margin-bottom: 20px; display: inline-block;">
                <img src="<?php echo $imgUrl; ?>" width="60" height="60" style="float: left;"/>
                <h2 style="float: right; vertical-align: middle; margin-top: 20px; font-size: 20px;">Flashchat App Settings</h2>
            </div>
            <div style ="margin-bottom: 40px;">
                <h2 style="font-size: 18px;">Setup and Settings</h2>
                <p style="font-size: 16px;"><strong>Step 1:</strong> Click <strong>WooCommerce Settings</strong> to activate <Strong>Woocommerce Legacy API</Strong> from <Strong>Woocommerce --> Settings --> Advanced tab --> Legacy API.</Strong></br>
                  Depending on your Woocommerce version, you might find this setting under <strong>Woocommerce --> Settings --> API --> enable the REST API</strong>.
                  </br>
                  </br>
                  <strong>Step 2:</strong> Click <strong>Start Integration</strong> to initiate integration and configure your Flashchat account.</p>
                </br>
                <a href="admin.php?page=wc-settings&tab=advanced&section=legacy_api"
                style ="font-size: 16px; background-color: #a36597; color: white; border: 1px solid #a36597; height: auto; padding: .5em .7em; text-decoration: none;" type="submit">
                WooCommerce Settings
                </a> &nbsp; &nbsp;
                <a 
                target="_blank"
                rel="noopener noreferrer"
                href="<?php echo $request_url?>"
                style ="font-size: 16px; background-color: #007aff; color: white; border: 1px solid #0087be; height: auto; padding: .5em .7em; text-decoration: none;" type="submit">
                Start Integration
                </a>
            </div>
            </br>
            <div>
                <p style="font-size: 16px;">After completing Flashchat integration, this field will be populated with your <strong>Facebook Page Id</strong>:</p>
                <table>
                <tr valign="top">
                <th scope="row"><label for="myplugin_option_name" style="color: #007aff; font-size: 16px;">Facebook Page Id:&nbsp; &nbsp;</label></th>
                <td><input readonly type="text" id="myplugin_option_name" name="myplugin_option_name" value="<?php echo get_option('account_id'); ?>" /></td>
                </tr>
                </table>
            </div>
        </div>
    <?php
    }

    // register settings page
    function flashchat_register_settings_page() {
        add_options_page('Settings', 'Flashchat App Settings', 'manage_options', 'flashchat_app', 'FlashchatPlugin::flashchat_settings_page');
    }

    // register flashchat wordpress REST API custom endpoints
    // flashchat/v1/cart/add for creating a cart and /flashchat/v1/cart for retriving current cart 
    function registerFlashchatCartEndpoints() {
        register_rest_route( 'flashchat/v1', 'cart/add/(?P<items>\d+:\d+:\d+(,\d+:\d+:\d+)*)/utm_source=(?P<utm_source>[a-zA-Z0-9 .\-\_]+)/utm_medium=(?P<utm_medium>[a-zA-Z0-9 .\-\_]+)/discount=(?P<discount>(.+)*)', array(
            'methods' => 'GET',
            'callback' => 'FlashchatPlugin::add_to_cart',
        ));
        register_rest_route( 'flashchat/v1', 'cart/', array(
            'methods' => 'GET',
            'callback' => 'FlashchatPlugin::get_cart',
        ));
    }

    // accepts digits in format product_id1:quantity1:variation_id1, product_id2:quantity2:variation_id2
    // as is defined in the endpoints regex
    // eg 29:1:0,30:2:44 would add product with id 29, quantity 1 and variation with id 44 of product with id 30, quantity 2 
    static function add_to_cart( $data ) {
        // empty cart
        WC()->cart->empty_cart();
        // Add item to cart using woocommerce WC_Cart class and funtions
        $items = explode(',', $data['items']);
        $utm_source = $data['utm_source'];
        $utm_medium = $data['utm_medium'];
        $discount = $data['discount'];
        
        foreach ($items as $item) {
            $prod_qnt_variation = explode(':', $item);
            if (count($prod_qnt_variation) === 2) {
                WC()->cart->add_to_cart( $prod_qnt_variation[0], $prod_qnt_variation[1], $variation_id, null, null );
            }
            else {
                WC()->cart->add_to_cart( $prod_qnt_variation[0], $prod_qnt_variation[1], $prod_qnt_variation[2], null, null );
            }
        }

        //add discount code
        if ($discount !== '') {
            WC()->cart->add_discount( sanitize_text_field( $discount ) );
        }

        //save utm metadata as session var to be added to order
        WC()->session->set(
            'utm_data',
            array(
               'utm_source' => $utm_source,
               'utm_medium' => $utm_medium));

        //redirect to /cart to show the new cart
        if (wp_redirect( home_url() . '/cart' )){
            exit;
        }
    }

    // retrieves current cart contents, cart_hash and flashchatWoocommerceSession which acts as customerId
    static function get_cart() {
        $customerId;
        if (isset($_COOKIE['flashchatWoocommerceSession'])) {
           $customerId = $_COOKIE['flashchatWoocommerceSession'];
        }
        else {
            $nonce = uniqid(mt_rand(), true);
            setcookie('flashchatWoocommerceSession', $nonce, time() + (86400 * 30), '/'); //86400 = 1 day
            $customerId = $nonce;
        }
        $cart_contents = WC()->cart->get_cart_contents();
        //cart hash as will be stored in Orders
        $cart_hash = md5( json_encode( wc_clean( WC()->cart->get_cart_for_session() ) ) . WC()->cart->total );
        return array('customerId' => $customerId, 'cartContents' => $cart_contents, 'cartHash'=> $cart_hash);
    }

    // add customerId as meta data WC_Order to be checked from /isOrderPlaced
    function before_checkout_create_order_add_customer_id( $order, $data ) {
        if (isset($_COOKIE['flashchatWoocommerceSession'])) {
            $order->update_meta_data( '_customerId', $_COOKIE['flashchatWoocommerceSession'] );
        }
        else {
            $nonce = uniqid(mt_rand(), true);
            setcookie('flashchatWoocommerceSession', $nonce, time() + (86400 * 30), '/'); //86400 = 1 day
            $order->update_meta_data( '_customerId', $nonce );
        }

        //save utm as metadata
        $session_var  = 'utm_data';
        $session_data = WC()->session->get($session_var);
        
        if(!empty($session_data)) {
            $order->update_meta_data( '_utmSource', $session_data['utm_source'] );
            $order->update_meta_data( '_utmMedium', $session_data['utm_medium'] );
            //wc_add_order_item_meta($item_id, $session_var, $session_data);
            WC()->session->__unset($session_var);
        }
    }

    /**
     * Handle a custom '_customerId' query var to get orders with the 'customvar' meta.
     * @param array $query - Args for WP_Query.
     * @param array $query_vars - Query vars from WC_Order_Query.
     * @return array modified $query
     */
    function handle_custom_query_var_customer_id( $query, $query_vars ) {
        if ( ! empty( $query_vars['_customerId'] ) ) {
            $query['meta_query'][] = array(
                'key' => '_customerId',
                'value' => esc_attr( $query_vars['_customerId'] ),
            );
        }
        return $query;
    }
}
