<?php
/**
 * Class that contains Woocommerce custom endpoints, extending WC_API_Resource
 **/
class WC_API_Flashchat extends WC_API_Resource {
    const PATH = '/flashchat';
    
    public function register_routes( $routes ) {
        //GET Request
        $routes[self::PATH . '/isOrderPlaced/(?P<customerid_and_cart_hash>.+:\w+)'] = array(
            array(array($this, 'is_order_placed'), WC_API_Server::READABLE),
        );

        //POST Request
        $routes[self::PATH . '/accountId/(?P<id>\d+)'] = array(
            array(array($this, 'storeAccountId'), WC_API_Server::CREATABLE | WC_API_Server::ACCEPT_DATA),
        );
        
		return $routes;
    }
    
    // check if an order is placed using customerId and cart hash 
    // received in format _customerId:cart_hash, for example: 123sjosu0:3129ajhojkk28kjyynnks
    public function is_order_placed($customerid_and_cart_hash) {
        $id_and_hash = explode(':', $customerid_and_cart_hash);
        $orders = wc_get_orders( array('_customerId' => $id_and_hash[0]));
        if (empty($orders)) {
            $response = new WP_REST_Response('No');
            $response->set_status(404);
            return $response;
        }
        else {
            foreach ($orders as $order) {
                $status = $order->get_status();
                if (($status === 'pending' || $status === 'processing' || $status === 'on-hold' )) {
                    $cart_hash = $order->get_cart_hash();
                    if ($cart_hash === $id_and_hash[1]) {
                        $response = new WP_REST_Response('Yes');
                        $response->set_status(200);
                        return $response;
                    }
                }
            }
            $response = new WP_REST_Response('No');
            $response->set_status(404);
            return $response;
        }
	}
    
    // save Flashchat account Id after Flashchat account creation
    public function storeAccountId($id, $data) {
        update_option( 'account_id', $id );
        $showtest = get_option('account_id');
        return new WP_REST_Response( $id, array( 'status' => 200 ) );
    }
}
